var babelHelpers = require("./babelHelpers");
/**
 * @file esui-react/Calendar
 * @author cxtom<cxtom2010@gmail.com>
 */

var _ = require('underscore');
var React = require('react');
var PropTypes = React.PropTypes;

var cx = require('melon-classname').create('Calendar');
var Icon = require('./Icon');
var Confirm = require('./dialog/Confirm');
var Panel = require('./calendar/Panel');

var DateTime = require('./common/util/date');
var Validity = require('./Validity');

var Calendar = React.createClass({

    displayName: 'Calendar',

    getInitialState() {

        return {

            // 缓存用户在 confirm 前的选中值
            date: this.parseDate(this.props.value),

            // 是否打开选择窗
            open: false
        };
    },

    componentWillReceiveProps(nextProps) {
        var value = nextProps.value;

        if (value !== this.props.value) {
            this.setState({
                date: this.parseDate(value)
            });
        }
    },

    /**
     * 格式化日期对象
     *
     * @param  {string} value 日期字符串
     * @return {Date}         转化后的日期对象
     * @private
     */
    parseValue(value) {
        return this.parseDate(value);
    },

    /**
     * 格式化日期
     *
     * @param {Date} rawValue 源日期对象
     * @param {string=} format 日期格式，默认为当前实例的dateFormat
     * @return {string} 格式化后的日期字符串
     * @private
     */
    stringifyValue(rawValue) {

        if (!_.isDate(rawValue)) {
            return rawValue;
        }

        var format = this.props.dateFormat.toLowerCase();

        return DateTime.format(rawValue, format, this.props.lang);
    },

    parseDate(date) {

        if (!_.isString(date)) {
            return date;
        }

        var format = this.props.dateFormat.toLowerCase();

        return DateTime.parse(date, format);
    },

    getValue() {
        return this.stringifyValue(this.props.value);
    },

    /**
     * 点击textbox时触发
     *
     * @private
     */
    onLabelClick() {

        if (this.props.disabled || this.props.readOnly) {
            return;
        }

        this.setState({ open: true });
    },

    /**
     * rawValue 在Calendar Dialog上点击确定或取消按钮触发
     *
     * @private
     */
    onConfirm() {
        var _state = this.state;
        var value = _state.value;
        var date = _state.date;

        if (DateTime.isEqualDate(date, value)) {
            this.setState({ open: false });
            return;
        }

        this.setState({
            open: false
        }, () => {
            this.props.onChange({
                type: 'change',
                target: this,
                value: this.stringifyValue(date)
            });
        });
    },

    onCancel() {
        this.setState({ open: false });
    },

    onDateChange(e) {
        var value = e.value;
        var autoConfirm = this.props.autoConfirm;

        this.setState({
            date: value
        }, () => {
            if (autoConfirm) {
                this.onConfirm();
            }
        });
    },

    render() {
        var state = this.state;
        var props = this.props;
        var lang = props.lang;
        var value = props.value;
        var disabled = props.disabled;
        var size = props.size;
        var name = props.name;
        var dateFormat = props.dateFormat;
        var validity = props.validity;
        var others = babelHelpers.objectWithoutProperties(props, ['lang', 'value', 'disabled', 'size', 'name', 'dateFormat', 'validity']);
        var begin = props.begin;
        var end = props.end;

        begin = this.parseDate(begin);
        end = this.parseDate(end);

        var open = state.open;
        var date = state.date;

        return React.createElement(
            'div',
            babelHelpers.extends({}, others, {
                className: cx(props).addStates({ focus: open }).build() }),
            React.createElement('input', {
                name: name,
                ref: 'input',
                type: 'hidden',
                value: value,
                disabled: disabled,
                size: size }),
            React.createElement(
                'label',
                { onClick: this.onLabelClick },
                DateTime.format(this.parseDate(value), dateFormat.toLowerCase(), lang),
                React.createElement(Icon, { icon: 'expand-more' })
            ),
            React.createElement(Validity, { validity: validity }),
            React.createElement(
                Confirm,
                {
                    open: open,
                    variants: ['calendar'],
                    onConfirm: this.onConfirm,
                    onCancel: this.onCancel,
                    size: size,
                    buttonVariants: ['secondery', 'calendar'] },
                React.createElement(Panel, {
                    date: date,
                    begin: begin,
                    end: end,
                    lang: lang,
                    onChange: this.onDateChange })
            )
        );
    }
});

Calendar.LANG = {

    // 对于 '周' 的称呼
    week: '周',

    // 星期对应的顺序表示
    days: '日,一,二,三,四,五,六'

};

Calendar.defaultProps = {
    value: DateTime.format(new Date(), 'yyyy-mm-dd', Calendar.LANG),
    dateFormat: 'yyyy-MM-dd',
    lang: Calendar.LANG,
    validateEvents: ['change']
};

Calendar.propTypes = {

    value: PropTypes.string,

    autoConfirm: PropTypes.bool,

    dateFormat: PropTypes.string,

    end: PropTypes.oneOfType([PropTypes.object, PropTypes.string]),

    begin: PropTypes.oneOfType([PropTypes.object, PropTypes.string]),

    onChange: PropTypes.func,

    lang: PropTypes.shape({
        week: PropTypes.string,
        days: PropTypes.string
    })

};

module.exports = require('./createInputComponent').create(Calendar);