var babelHelpers = require("./babelHelpers");
/**
 * @file Form
 * @author leon(ludafa@outlook.com)
 */

var React = require('react');
var validator = require('./Validator');

var PropTypes = React.PropTypes;

var Form = React.createClass({

    displayName: 'Form',

    propTypes: {
        onSumbit: PropTypes.func,
        target: PropTypes.string,
        action: PropTypes.string,
        method: PropTypes.oneOf(['POST', 'GET']),
        validator: PropTypes.shape({
            validate: PropTypes.func.isRequired
        })
    },

    getDefaultProps() {
        return {
            validator: validator
        };
    },

    getInitialState() {
        this.fields = [];
        return {};
    },

    childContextTypes: {
        attachForm: PropTypes.func,
        detachForm: PropTypes.func,
        validator: PropTypes.shape({
            validate: PropTypes.func.isRequired
        }),
        pointer: PropTypes.string.isRequired
    },

    getChildContext() {
        return {
            pointer: '/',
            attachForm: this.addField,
            detachForm: this.removeField,
            validator: this.props.validator
        };
    },

    componentWillUnmount() {
        this.fields.length = 0;
        this.fields = null;
    },

    addField(field) {
        this.fields.push(field);
    },

    removeField(field) {
        var fields = this.fields;

        if (fields) {
            this.fields = this.fields.filter(f => {
                return f !== field;
            });
        }
    },

    isValidFormField(field) {

        var value = field.getValue();
        var pointer = field.pointer;
        var props = field.props;
        var name = props.name;
        var disabled = props.disabled;

        return name && !disabled && value != null && pointer && pointer.lastIndexOf('/') === 0;
    },

    getData() {
        return this.fields.reduce((data, field) => {

            if (this.isValidFormField(field)) {
                data[field.props.name] = field.getValue();
            }

            return data;
        }, {});
    },

    validate() {
        return this.checkValidity().isValid;
    },

    checkValidity() {
        return this.fields.reduce((formValidity, field) => {

            // 不校验以下字段
            if (!this.isValidFormField(field)) {
                return formValidity;
            }

            var value = field.getValue();
            var validity = field.validate(value);

            return {
                isValid: formValidity.isValid && validity.isValid(),
                errors: [].concat(babelHelpers.toConsumableArray(formValidity.errors), babelHelpers.toConsumableArray(validity.states.filter(state => !state.isValid)))
            };
        }, {
            isValid: true,
            errors: []
        });
    },

    onSubmit(e) {
        var _props = this.props;
        var onSubmit = _props.onSubmit;
        var noValidate = _props.noValidate;

        if (!noValidate) {
            if (!this.validate()) {
                e.preventDefault();
                return;
            }
        }

        if (onSubmit) {
            e.data = this.getData();
            onSubmit(e);
        }
    },

    render() {
        var props = this.props;

        return React.createElement('form', babelHelpers.extends({}, props, { onSubmit: this.onSubmit }));
    }

});

module.exports = Form;