var babelHelpers = require("./babelHelpers");
/**
 * @file esui-react/RangeCalendar
 * @author cxtom <cxtom2010@gmail.com>
 * @author leon <ludafa@outlook.com>
 */

var _ = require('underscore');
var React = require('react');
var Calendar = require('./Calendar');
var Icon = require('./Icon');
var Confirm = require('./dialog/Confirm');
var Panel = require('./calendar/Panel');
var Validity = require('./Validity');

var DateTime = require('./common/util/date');
var cx = require('melon-classname').create('RangeCalendar');

var RangeCalendar = React.createClass({

    displayName: 'RangeCalendar',

    getInitialState() {

        return {
            open: false,
            date: this.getNormalizeValue(this.props)
        };
    },

    componentWillReceiveProps(nextProps) {
        var value = nextProps.value;

        if (value !== this.props.value) {
            this.setState({
                date: this.getNormalizeValue(nextProps)
            });
        }
    },

    getNormalizeValue(props) {
        var begin = props.begin;
        var end = props.end;
        var value = props.value;

        begin = this.parseDate(begin);
        end = this.parseDate(end);

        var valueBegin = this.parseDate(value[0]);
        var valueEnd = this.parseDate(value[1]);

        // 这里我们需要一个全新的 value
        value = [valueBegin && DateTime.isAfterDate(begin, valueBegin) ? begin : valueBegin, valueEnd && DateTime.isBeforeDate(end, valueEnd) ? end : valueEnd];

        // 下边这种做法是错误的，不能直接修改 props 中的值
        // value[0] = _.isDate(begin) && DateTime.isAfterDate(begin, value[0]) ? begin : value[0];
        // value[1] = _.isDate(end) && DateTime.isBeforeDate(end, value[1]) ? end : value[1];

        return value;
    },

    getValue() {
        return this.getNormalizeValue(this.props).map(date => {
            return this.formatDate(date);
        });
    },

    /**
     * 点击textbox时触发
     *
     * @private
     */
    onLabelClick() {
        var _props = this.props;
        var disabled = _props.disabled;
        var readOnly = _props.readOnly;

        if (disabled || readOnly) {
            return;
        }

        this.setState({ open: true });
    },

    /**
     * Calendar DialogCalendar隐藏时触发
     *
     * @private
     */
    onCancel() {
        this.setState({
            open: false
        });
    },

    onDateChange(index, e) {
        var value = e.value;

        var date = [].concat(babelHelpers.toConsumableArray(this.state.date));

        date[index] = value;

        this.setState({
            date: date,
            month: date
        });
    },

    onConfirm() {
        var date = this.state.date;
        var _props2 = this.props;
        var value = _props2.value;
        var onChange = _props2.onChange;

        // 不管怎么样，先把窗口关了

        this.setState({
            open: false
        }, () => {

            // 如果值发生了变化，那么释放一个 change 事件
            if (!DateTime.isEqualDate(date[0], this.parseDate(value[0])) || !DateTime.isEqualDate(date[1], this.parseDate(value[1]))) {
                onChange({
                    type: 'change',
                    target: this,
                    value: date.map(this.formatDate)
                });
            }
        });
    },

    /**
     * 按设置格式化日期
     *
     * @param {Date} date 日期
     * @return {string}
     */
    formatDate(date) {
        var _props3 = this.props;
        var dateFormat = _props3.dateFormat;
        var lang = _props3.lang;

        return DateTime.format(date, dateFormat.toLowerCase(), lang);
    },

    parseDate(date) {

        if (!_.isString(date)) {
            return date;
        }

        var format = this.props.dateFormat.toLowerCase();

        return DateTime.parse(date, format);
    },

    render() {

        var props = this.props;

        var lang = props.lang;
        var disabled = props.disabled;
        var size = props.size;
        var dateFormat = props.dateFormat;
        var name = props.name;
        var begin = props.begin;
        var end = props.end;
        var validity = props.validity;
        var others = babelHelpers.objectWithoutProperties(props, ['lang', 'disabled', 'size', 'dateFormat', 'name', 'begin', 'end', 'validity']);

        var value = this.getValue();

        var _state = this.state;
        var open = _state.open;
        var date = _state.date;

        begin = this.parseDate(begin);
        end = this.parseDate(end);

        return React.createElement(
            'div',
            babelHelpers.extends({}, others, {
                className: cx(props).addStates({ focus: open }).build() }),
            React.createElement('input', {
                name: name,
                ref: 'input',
                type: 'hidden',
                value: value.join(','),
                disabled: disabled }),
            React.createElement(
                'label',
                { onClick: this.onLabelClick },
                `${ value[0] } 至 ${ value[1] }`,
                React.createElement(Icon, { icon: 'expand-more' })
            ),
            React.createElement(Validity, { validity: validity }),
            React.createElement(
                Confirm,
                {
                    open: open,
                    variants: ['calendar'],
                    onConfirm: this.onConfirm,
                    onCancel: this.onCancel,
                    size: size,
                    buttonVariants: ['secondery', 'calendar'] },
                React.createElement(
                    'div',
                    { className: cx().part('row').build() },
                    React.createElement(Panel, {
                        lang: lang,
                        date: date[0],
                        begin: begin,
                        end: date[1],
                        onChange: this.onDateChange.bind(this, 0) }),
                    React.createElement(Panel, {
                        lang: lang,
                        date: date[1],
                        begin: date[0],
                        end: end,
                        onChange: this.onDateChange.bind(this, 1) })
                )
            )
        );
    }

});

RangeCalendar.defaultProps = babelHelpers.extends({}, Calendar.defaultProps, {
    value: [DateTime.format(new Date(), 'yyyy-mm-dd', Calendar.LANG), DateTime.format(DateTime.addMonths(new Date(), 1), 'yyyy-mm-dd', Calendar.LANG)]
});

var PropTypes = React.PropTypes;

RangeCalendar.propTypes = babelHelpers.extends({}, Calendar.propTypes, {
    value: PropTypes.arrayOf(PropTypes.string),
    autoOk: PropTypes.bool,
    dateFormat: PropTypes.string,
    begin: PropTypes.oneOfType([PropTypes.object, PropTypes.string]),
    end: PropTypes.oneOfType([PropTypes.object, PropTypes.string])
});

module.exports = require('./createInputComponent').create(RangeCalendar);