/**
 * @file melon/Uploader
 * @author leon(ludafa@outlook.com)
 */

var React = require('react');
var Button = require('./Button');
var Icon = require('./Icon');
var Progress = require('./Progress');
var Link = require('./Link');
var cx = require('melon-classname').create('Uploader');
var Validity = require('./Validity');

var Uploader = React.createClass({

    displayName: 'Uploader',

    getInitialState() {

        return {
            isUploading: false,
            isUploaded: !!this.props.value
        };
    },

    onSelect() {
        this.refs.file.click();
    },

    onFileChange(e) {

        this.setUploading();

        this.props.upload({
            target: this,
            files: e.target.files
        }).then(result => {
            this.setFile(result);
        }, error => {
            this.clearFile();
        });
    },

    onClearClick() {
        this.clearFile();
    },

    setUploading() {
        this.setState({
            isUploading: true
        });
    },

    setFile(value) {

        this.setState({
            value,
            isUploaded: true,
            isUploading: false
        }, () => {
            this.props.onChange({
                type: 'change',
                target: this,
                value: value
            });
        });
    },

    clearFile() {

        this.setState({
            rawValue: '',
            isUploaded: false,
            isUploading: false
        }, () => {
            this.props.onChange({
                type: 'change',
                target: this,
                value: ''
            });
        });
    },

    renderUploadFile() {
        var _state = this.state;
        var isUploading = _state.isUploading;
        var isUploaded = _state.isUploaded;

        return isUploading || isUploaded ? null : React.createElement('input', {
            ref: 'file',
            type: 'file',
            className: cx().part('file').build(),
            onChange: this.onFileChange,
            accept: this.props.accept });
    },

    renderUploadButton() {
        var _state2 = this.state;
        var isUploading = _state2.isUploading;
        var isUploaded = _state2.isUploaded;
        var value = _state2.value;
        var size = this.props.size;

        if (isUploading) {
            return React.createElement(Progress, {
                size: size,
                mode: 'indeterminate',
                shape: 'circle' });
        }

        if (isUploaded) {

            return React.createElement(
                'div',
                { className: cx().part('uploaded').build() },
                React.createElement(Icon, { icon: 'done', size: size }),
                ' 已上传',
                React.createElement(
                    Link,
                    {
                        size: size,
                        href: value,
                        variants: ['button'],
                        target: '_blank' },
                    '查看'
                ),
                React.createElement(
                    Button,
                    {
                        size: size,
                        type: 'button',
                        onClick: this.onClearClick },
                    '重选'
                )
            );
        }

        return React.createElement(
            Button,
            {
                type: 'button',
                variants: ['raised'],
                onClick: this.onSelect },
            React.createElement(Icon, { icon: 'file-upload' }),
            '点击上传'
        );
    },

    render() {

        var props = this.props;
        var value = props.value;
        var validity = props.validity;

        return React.createElement(
            'div',
            { className: cx(props).build() },
            React.createElement('input', { name: props.name, type: 'hidden', value: value }),
            this.renderUploadFile(),
            this.renderUploadButton(),
            React.createElement(Validity, { validity: validity })
        );
    }

});

var PropTypes = React.PropTypes;

Uploader.propTypes = {
    multiple: PropTypes.bool,
    accept: PropTypes.string,
    files: PropTypes.array,
    upload: PropTypes.func.isRequired
};

Uploader.defaultProps = {
    validateEvents: ['change']
};

module.exports = require('./createInputComponent').create(Uploader);