/**
 * @file Validitor
 * @author leon(ludafa@outlook.com)
 */

var Validity = require('./validator/Validity');

function Validator() {
    this.rules = [];
}

Validator.prototype.addRule = function (rule) {
    this.rules.push(rule);
    return this;
};

Validator.prototype.resolveCheckers = function () {
    var config = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
    var rules = this.rules;

    return rules.reduce((activeCheckers, checker) => {
        var name = checker.name;
        var check = checker.check;

        if (name in config) {
            activeCheckers.push({
                name,
                check: check,
                value: config[name]
            });
        }

        return activeCheckers;
    }, []);
};

Validator.prototype.validate = function (value, component) {

    return this.resolveCheckers(component.props.rules).reduce((validity, checker) => {
        var check = checker.check;

        var state = check(value, component);
        validity.addState(state);
        return validity;
    }, new Validity());
};

Validator.prototype.createCustomValidity = function (customValidity) {
    var validity = new Validity();
    validity.addState({
        isValid: false,
        message: customValidity
    });
    return validity;
};

var validator = new Validator();

validator.create = () => {
    return new Validator();
};

validator.addRule({
    name: 'required',
    check: (value, component) => {
        var requiredErrorMessage = component.props.rules.requiredErrorMessage;

        var isValid = value instanceof Array ? value.length : typeof value === 'string' ? value !== '' : value != null;

        return {
            isValid: isValid,
            message: requiredErrorMessage || `请填写此字段`
        };
    }
});

module.exports = validator;