var babelHelpers = require("../babelHelpers");
/**
 * @file melon/select/SeparatePopup
 * @author leon(ludafa@outlook.com)
 */

var React = require('react');
var cx = require('melon-classname').create('SeparatePopup');

var _require = require('react-motion');

var Motion = _require.Motion;
var spring = _require.spring;

var domUtil = require('../common/util/dom');

var SelectSeparatePopup = React.createClass({

    displayName: 'SelectSeparatePopup',

    getInitialState() {

        // 做一个可以随时释放的 debounce 啦
        this.onWindowResize = (() => {

            var handler = this.onWindowResize;

            return () => {
                clearTimeout(this.windowResizeTimer);
                this.windowResizeTimer = setTimeout(handler, 500);
            };
        })();

        this.id = Date.now();

        return {
            styles: {
                top: 0,
                left: -5000,
                height: 0,
                opacity: 0,
                width: 0
            }
        };
    },

    componentDidMount() {
        domUtil.on(document.body, 'click', this.onClick);
    },

    componentWillReceiveProps(nextProps) {
        var open = nextProps.open;

        domUtil[open ? 'on' : 'off'](window, 'resize', this.onWindowResize);

        this.setState(babelHelpers.extends({}, this.state, {
            styles: this.getStyle(open)
        }));
    },

    componentWillUnmount() {
        domUtil.off(window, 'resize', this.onWindowResize);
        domUtil.off(document.body, 'click', this.onClick);
    },

    getStyle(open) {

        if (!open) {
            return babelHelpers.extends({}, this.state.styles, {
                height: 0,
                opacity: 0
            });
        }

        var target = this.props.target;
        var main = this.main;

        var targetPosition = domUtil.getPosition(target);

        var _main$style = main.style;
        var top = _main$style.top;
        var left = _main$style.left;
        var width = _main$style.width;
        var height = _main$style.height;
        var overflow = _main$style.overflow;

        // 先把这个货弄走，展开计算它的宽度和高度

        main.style.top = '0';
        main.style.left = '-5000px';
        main.style.overflow = 'visible';
        main.style.height = 'auto';
        main.style.width = 'auto';

        var popupPosition = domUtil.getPosition(main);

        main.style.top = top;
        main.style.left = left;
        main.style.overflow = overflow;
        main.style.height = height;
        main.style.width = width;

        var scrollTop = domUtil.getScrollTop();
        var scrollLeft = domUtil.getScrollLeft();
        var clientWidth = domUtil.getClientWidth();
        var clientHeight = domUtil.getClientHeight();

        var rTop = undefined;
        var rLeft = undefined;

        // 首先计算垂直位置
        // 如垂直位置从目标的上边缘开始，如果加上浮层高度，未溢出视野，那么就取目标上边缘
        if (targetPosition.top + popupPosition.height < scrollTop + clientHeight) {
            rTop = targetPosition.top;
        }
        // 否则尽量贴在视野的底边上，如果此时浮层顶部溢出了视野，那说明视野就是太小了。。。
        else {
                rTop = clientHeight + scrollTop - popupPosition.height - 20;
            }

        // 然后计算水平位置
        // 尝试以目标的左边缘开始，如果加上浮层宽宽未溢出视野，那么取目标左边缘
        if (targetPosition.left + popupPosition.width < clientWidth + scrollLeft) {
            rLeft = targetPosition.left;
        }
        // 否则尽量贴在视野的右边，如果此时浮层左侧溢出视野，那说明视野就是太小了。
        else {
                rLeft = clientWidth + scrollLeft - popupPosition.width - 20;
            }

        return {
            opacity: 1,
            top: rTop,
            left: rLeft,
            height: popupPosition.height,
            width: Math.max(targetPosition.width, popupPosition.width)
        };
    },

    onClick(e) {
        var target = e.target;
        var main = this.main;
        var _props = this.props;
        var onHide = _props.onHide;
        var open = _props.open;

        if (open && main !== target && !domUtil.contains(main, target)) {
            onHide && onHide();
        }
    },

    onWindowResize() {
        this.setState(babelHelpers.extends({}, this.state, {
            styles: this.getStyle(true)
        }));
    },

    render() {
        var children = this.props.children;

        var className = cx(this.props).build();
        var contentClassName = cx().part('content').build();

        var styles = this.state.styles;
        var height = styles.height;
        var opacity = styles.opacity;

        return React.createElement(
            Motion,
            {
                style: babelHelpers.extends({}, styles, {
                    height: spring(height, [120, 15]),
                    opacity: spring(opacity, [120, 15])
                }) },
            style => {

                return React.createElement(
                    'div',
                    {
                        className: className,
                        style: babelHelpers.extends({}, style, {
                            visibility: style.opacity < 0.1 ? 'hidden' : 'visible'
                        }),
                        ref: main => {
                            this.main = main;
                        } },
                    React.createElement(
                        'div',
                        { className: contentClassName },
                        children
                    )
                );
            }
        );
    }

});

var PropTypes = React.PropTypes;

SelectSeparatePopup.propTypes = {
    target: PropTypes.object.isRequired,
    onHide: PropTypes.func.isRequired
};

module.exports = SelectSeparatePopup;